#ifndef __C_GUI_TTFONT_H_INCLUDED__
#define __C_GUI_TTFONT_H_INCLUDED__

#include "IrrCompileConfig.h"
#ifdef _IRR_COMPILE_WITH_GUI_

#include "ft2build.h"
#include "freetype/freetype.h"
#include "IGUIFont.h"
#include "irrArray.h"

// >> Add by uirou for Multilingual start
#ifdef USE_ICONV
#include <langinfo.h>
#include <iconv.h>
#endif
// << Add by uirou for Multilingual end

namespace irr
{

namespace video
{
	class IVideoDriver;
	class ITexture;
}

namespace scene
{
	class ISceneNode;
	class ISceneManager;
}

namespace gui
{
	class IGUIEnvironment;

class CGUITTFace : public IReferenceCounted
{
public:
	CGUITTFace();
	virtual ~CGUITTFace();
	bool loaded;
	FT_Library	library;
	FT_Face		face;
// >> Add by uirou for Multilingual start
#ifdef USE_ICONV
	iconv_t cd;
#endif
// << Add by uirou for Multilingual end
	bool load(const c8* filename);
};
class CGUITTGlyph : public IReferenceCounted
{
public:
	bool cached;
	video::IVideoDriver* Driver;
	CGUITTGlyph();
	virtual ~CGUITTGlyph();
// >> Add solehome's code for memory access error START
	void init();
// << Add solehome's code for memory access error END
	void cache(u32 idx);
	FT_Face *face;
	u32 size;
	u32 top;
	u32 left;
	u32 texw;
	u32 texh;
	u32 imgw;
	u32 imgh;
	video::ITexture *tex;
	u32 top16;
	u32 left16;
	u32 texw16;
	u32 texh16;
	u32 imgw16;
	u32 imgh16;
	video::ITexture *tex16;
	s32 offset;
	u8 *image;
};
class CGUITTFont : public IGUIFont
{
public:
	u32 size;

	//! constructor
	CGUITTFont(IGUIEnvironment *env);

	//! destructor
	virtual ~CGUITTFont();

	//! loads a truetype font file
	bool attach(CGUITTFace *Face,u32 size);

	//! Draws an text and clips it to the specified rectangle if wanted.
	/** \param text: Text to draw
	 \param position: Rectangle specifying position where to draw the text.
	 \param color: Color of the text
	 \param hcenter: Specifiies if the text should be centered horizontally into the rectangle.
	 \param vcenter: Specifiies if the text should be centered vertically into the rectangle.
	 \param clip: Optional pointer to a rectangle against which the text will be clipped.
	 If the pointer is null, no clipping will be done. */
	virtual void draw(const wchar_t* text, const core::rect<s32>& position,
		video::SColor color, bool hcenter=false, bool vcenter=false,
		const core::rect<s32>* clip=0);

	//! Calculates the dimension of a text.
	/** \return Returns width and height of the area covered by the text if it would be
	  drawn. */
	virtual core::dimension2d<s32> getDimension(const wchar_t* text) const;

	//! Calculates the index of the character in the text which is on a specific position.
	/** \param text: Text string.
	\param pixel_x: X pixel position of which the index of the character will be returned.
	\return Returns zero based index of the character in the text, and -1 if no no character
	is on this position. (=the text is too short). */
	virtual s32 getCharacterFromPos(const wchar_t* text, s32 pixel_x) const;

	//! Sets global kerning for the font.
	virtual void setKerningWidth (s32 kerning);
	virtual void setKerningHeight (s32 kerning);

	//! Gets kerning values (distance between letters) for the font. If no parameters are provided,
	/** the global kerning distance is returned.
	\param thisLetter: If this parameter is provided, the left side kerning for this letter is added
	to the global kerning value. For example, a space might only be one pixel wide, but it may
	be displayed as several pixels.
	\param previousLetter: If provided, kerning is calculated for both letters and added to the global
	kerning value. For example, in a font which supports kerning pairs a string such as 'Wo' may have 
	the 'o' tucked neatly under the 'W'.
	*/
	virtual s32 getKerningWidth(const wchar_t* thisLetter=0, const wchar_t* previousLetter=0) const;

	//! Returns the distance between letters
	virtual s32 getKerningHeight() const;

	scene::ISceneNode *createBillboard(const wchar_t* text,scene::ISceneManager *scene,scene::ISceneNode *parent = 0,s32 id = -1);

	bool AntiAlias;
	bool TransParency;
	bool attached;
private:
	s32 getWidthFromCharacter(const wchar_t c) const;
	u32 getGlyphByChar(const wchar_t c) const;
	video::IVideoDriver* Driver;
	mutable core::array< CGUITTGlyph > Glyphs;
	CGUITTFace *tt_face;
// >> Add by MadHyde for Ver.1.3 new functions begin
	s32 GlobalKerningWidth, GlobalKerningHeight;
// << Add by MadHyde for Ver.1.3 new functions end
};

} // end namespace gui
} // end namespace irr

#endif // _IRR_COMPILE_WITH_GUI_
#endif // __C_GUI_TTFONT_H_INCLUDED__

